-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...
local hbdp = LibStub("HereBeDragons-Pins-1.0")

-- string colors
local AddColor = addonTable.AddColor

--[[ map functions ]]--

local lists =  DailyGlobalCheck.ListFunctions
local maps  = DailyGlobalCheck.MapFunctions

local dgcicon = "Interface\\icons\\INV_Sigil_Freya"

local frames_pool = {}
local buttons = { worldmap = { p = CreateFrame("Frame", nil, WorldMapButton), pool = {} },
                  minimap  = { pool = {} } }
local quests_by_mapid = {}

CreateFrame("GameTooltip","DailyGlobalCheck_maptooltip", nil, "GameTooltipTemplate")
DailyGlobalCheck_maptooltip:SetFrameStrata("TOOLTIP")
DailyGlobalCheck_maptooltip:SetScale(0.8)

-- frame pool
local function getbtn()
 local btn = frames_pool[1]
 if btn then table.remove(frames_pool, 1) end
 return btn
end

local function freebtn(btn)
 table.insert(frames_pool, btn)
 btn:Hide()
end
--

-- events
local function mapbutton_onmouseup(self, mousebutton)
 if not IsControlKeyDown() or mousebutton ~= "RightButton" then
  local p = self:GetParent()
  if self:GetParent() == buttons.worldmap.p then
   WorldMapButton_OnClick(WorldMapButton, mousebutton)
  else
   Minimap_OnClick(Minimap, mousebutton)
  end
  return
 end
 
 local data = DailyGlobalCheck:getQData(self.questID)

 if data then
  local x = data[5][data[6]][1] / 100
  local y = data[5][data[6]][2] / 100
  addonTable.setTomTomWP(data[6], nil, x, y, { title = addonTable.get(data, 2)})
 end
end

local function mapbutton_onenter(self)
 local data = DailyGlobalCheck:getQData(self.questID)
 if not data then return end
 
 DailyGlobalCheck_maptooltip:SetOwner(self,"ANCHOR_BOTTOM")
 DailyGlobalCheck_maptooltip:ClearLines()
 DailyGlobalCheck_maptooltip:AddLine("|cff00AAFFDaily Global Check|r")
 DailyGlobalCheck_maptooltip:AddLine(AddColor(self.title or "","LIGHT_BLUE"))
 DailyGlobalCheck_maptooltip:AddLine(addonTable.get(data, 3).." "..addonTable.get(data, 2))
 DailyGlobalCheck_maptooltip:AddLine(addonTable.get(data, 4))
 DailyGlobalCheck_maptooltip:Show()
end

local function mapbutton_onleave()
 DailyGlobalCheck_maptooltip:Hide()
end
--

--- WorldMap and Minimap functions ---
local function createButton(t, questID, list)

 local f = getbtn()
 
 if not f then
  f = CreateFrame("FRAME")
  f:SetScript("OnMouseUp", mapbutton_onmouseup) 
  f:SetScript("OnEnter", mapbutton_onenter)
  f:SetScript("OnLeave", mapbutton_onleave)
  f:EnableMouse(DailyGlobalCheck_Options.UI[t.."_icon_interact"])
  f.tex = f:CreateTexture()
  f.tex:SetAllPoints()
 end

 f:SetParent(t == "worldmap" and buttons[t].p or nil)
 f.questID = questID
 f.title   = list and list["Title"]
 buttons[t].pool[questID] = f
 return f
end

function maps:GetButton(t, questID)
 return buttons[t] and buttons[t].pool[questID] or nil
end

function maps:AddButton(t, questID, list)
 local v = DailyGlobalCheck:getQData(questID)

 if v and v[5] and v[5][v[6]] then
  local f = maps:GetButton(t, questID) or createButton(t, questID, list)

  if not quests_by_mapid[v[6]] then quests_by_mapid[v[6]] = {} end
  if not tContains(quests_by_mapid[v[6]], questID) then table.insert(quests_by_mapid[v[6]], questID) end

  local x = v[5][v[6]][1]
  local y = v[5][v[6]][2]
  local size = DailyGlobalCheck_Options.UI[t.."_icon_size"]
  f:SetSize(size, size)
  
  local plugin = list or select(2, lists:Find(v[11]))
  f.tex:SetTexture(v[8] or ((plugin and plugin["Icon"]) or dgcicon),1)
  if t == "minimap" then
   hbdp:AddMinimapIconMF("DailyGlobalCheck", f, v[6], nil, x / 100, y / 100)
  else
   hbdp:AddWorldMapIconMF("DailyGlobalCheck", f, v[6], nil, x / 100, y / 100)
  end
 end
end

function maps:AddList(t, list)
 if not lists:isList(list) then return end
 
  for _, page in pairs(list["Order"]) do
   for _, group in pairs(page) do
    for i = 2, #group do
	 if not DailyGlobalCheck.isquestcompleted(group[i], true, list) and addonTable.isshown(group[i]) then
      maps:AddButton(t, group[i], list)
	 else
	  maps:RemoveButton(t, group[i])
	 end
	end
   end
  end
end

function maps:RemoveButton(t, questID)
 local btn = maps:GetButton(t, questID)

 if btn then
  freebtn(btn)
  buttons[t].pool[questID] = nil
  if t == "minimap" then
   hbdp:RemoveMinimapIcon("DailyGlobalCheck", btn)
  else
   hbdp:RemoveWorldMapIcon("DailyGlobalCheck", btn)
  end
 end
end

function maps:RemoveList(t, list)
 if not lists:isList(list) then return end
 
 for _, page in pairs(list["Order"]) do
  for _, group in pairs(page) do
   for i = 2, #group do
    maps:RemoveButton(t, group[i])
   end
  end
 end
end
---------------------------------

function maps:UpdateList(t, list)

 if t == "worldmap" and not DailyGlobalCheck_Options["show_mapicons"] then
  hbdp:RemoveAllWorldMapIcons("DailyGlobalCheck")
  return
 end

 if t == "minimap" and  not DailyGlobalCheck_Options["show_minimapicons"] then
  hbdp:RemoveAllMinimapIcons("DailyGlobalCheck")
  return
 end

 if lists:isActive(list) and lists:getShowMapIcons(list, t) then
  maps:AddList(t, list)
 else
  maps:RemoveList(t, list)
 end
end

function maps:UpdateIcons(t, list)

 if t == "worldmap" and not DailyGlobalCheck_Options["show_mapicons"] then
  hbdp:RemoveAllWorldMapIcons("DailyGlobalCheck")
  return
 end

 if t == "minimap" and  not DailyGlobalCheck_Options["show_minimapicons"] then
  hbdp:RemoveAllMinimapIcons("DailyGlobalCheck")
  return
 end

 if list and lists:isList(list) then
  maps:UpdateList(t, list)
 else
  DailyGlobalCheck:foreachLists(function(_,v)
   maps:UpdateList(t, v)
  end, false)
 end
end

function maps:UpdateIconsSize(t)
 if not buttons[t] then return end
 
 local size = DailyGlobalCheck_Options.UI[t.."_icon_size"]
 for k,v in pairs(buttons[t].pool) do
  v:SetSize(size, size)
 end
end

function maps:UpdateInteractable(t)
 if not buttons[t] then return end

 for k,v in pairs(buttons[t].pool) do
  v:EnableMouse(DailyGlobalCheck_Options.UI[t.."_icon_interact"])
 end
end

DGCEventFrame:RegisterEvent("WORLD_MAP_UPDATE")

local function checkMap(v)
 if not v then return end
 
 local i = 1
 while i < #v do
  if DailyGlobalCheck.isquestcompleted(v[i], true) then
   maps:RemoveButton("worldmap", v[i])
   maps:RemoveButton("minimap", v[i])
   table.remove(v, i)
   i = i - 1
  end
  i = i + 1
 end
end

local last_check = 0
function DGCEventFrame.WORLD_MAP_UPDATE(...)
 -- general check every 2~ minutes to be sure we hide completed quests' icons
 local now = time()
 if now > last_check + 120 then
  for k,v in pairs(quests_by_mapid) do
   checkMap(v)
  end
  last_check = now
  return
 end

 -- minimal check only for the current zone with every update
 local map = GetCurrentMapAreaID()
 local v = map and quests_by_mapid[map] or nil
 checkMap(v)
end